!
!svn $Id: ecodynamo_mod.h 1411 2015-07-09 23:02:13Z Pedro $
!================================================== Pedro Duarte     ===
!                                                                      !
!=======================================================================
!                                                                      !
!  Parameters for Powell et al. (2006) ecosystem model:                !
!                                                                      !
!  AttPhy    Light attenuation due to phytoplankton (self-shading      !
!              coefficient), [m2/millimole_N].                         !
!  AttSW     Light attenuation due to sea water, [1/m].                !
!  BioIter   Maximum number of iterations to achieve convergence of    !
!              the nonlinear solution.                                 !
!  BioIni    Initial concentration for analytical initial (uniform)    !
!              conditions.                                             !
!  DetRR     Detritus remineraliztion rate, [1/day].                   !
!  K_NO3     Half-saturation for phytoplankton nitrate uptake          !
!  Chl2C_m   Maximum chlorophyll to carbon ratio                       !
!              [mg Chl/mg C]                                           !
!              [millimole_N m-3].                                      !
!  Ivlev     Ivlev constant for zooplankton grazing parameterization,  !
!              [nondimensional].                                       !
!  PARfrac   Fraction of shortwave radiation that is available for     !
!              photosyntesis [nondimensional].                         !
!  PhyIS     Phytoplankton, initial slope of the P-I curve             ! 
!              [mg C mg C-1 h-1 microM photons m2 s].                  !
!  Beta      Phytoplankton, negative slope of the P-I curve            ! 
!              [mg C mg C-1 h-1 microM photons m2 s].                  !
!  PhyMRD    Phytoplankton mortality rate to the Detritus pool,        !
!              [1/day].                                                !
!  PhyMRN    Phytoplankton mortality rate to the Nitrogen pool,        !
!              [1/day].                                                !
!  Vm_NO3    Nitrate uptake rate, [1/day].                             !
!  wDet      Detrital sinking rate, [m/day].                           !
!  wPhy      Phytoplankton sinking rate, [m/day].                      !
!                                                                      !  
!******************EcoDynamo added stuff*******************************!
!  Pmax      Maximum photosynthetic rate [1/h]                         !
!  Iopt       Optimal light intensity [microEisntein/m2/s]             !
!  ImaxE      Saturation light intensity [microEisntein/m2/s]           ! 
!  AEiler    Parameter of the Eilers & Peeters equation                !
!            [microEinstein/m2/s]                                      !
!  BEiler    Parameter of the Eilers & Peeters equation                !
!            [microEinstein/m2/s]                                      !
!  CEiler    Parameter of the Eilers & Peeters equation                !
!            [microEinstein/m2/s]                                      !
!  MaintResp Maintenance respiration [mmol O2/mg Chl/h]                !
!  RespCoeff Respiration coefficient [nondimensional]                  !
!  DocLoss   Fraction of GPP lost as dissolved organics                !
!            [nondimensional]                                          ! 
!  DocStressLoss Fraction of GPP lost as dissolved organics            !
!            under nutriebnt stress [nondimensional]                   ! 
!  RedfieldC Redfield "default" mass of C per mass of P [gC/gP]        !
!  RedfieldN Redfield "default" mass of N per mass of P [gN/gP]        !
!  RedfieldP Redfield "default" mass of P                              !
!  TemperatureAugmentationRate  Exponent of the temperature limiting   !
!            function [1/degrees]                                      !
!  RatioLightDarkRespiration Ratio between respiration in the light    !
!            and respiration in the darkness [nondimensional]          !
!  minNPRatio Minimal N:P cell ratio [mgN/mgP]                         !
!  maxNPRatio Maximal N:P cell ratio [mgN/mgP]                         !
!  PMaxUptake Maximal phosphorus uptake [1/h]                          !
!  NMaxUptake Maximal nitrogen uptake [1/h]                            !   
!  KP         Half-saturation constant for P uptake [microM]           !
!  KNH4        Half-saturation constant for ammonia uptake [microM]     !
!  MinPCellQuota Minimal P cell contents [mgP/mgC]                     !
!  MaxPCellQuota Maximal P cell contents [mgP/mgC]                     !
!  MinNCellQuota Minimal N cell contents [mgP/mgC]                     !
!  MaxNCellQuota Maximal N cell contents [mgP/mgC]                     !
!  KPInternal    Half saturation constant for internal P limitation    !
!                [mgP/mgC]                                             !
!  KNInternal    Half saturation constant for internal N limitation    !
!                [mgN/mgC]                                             !
!  CarbonToOxygenProd Mass ratio between carbon fixed and oxygen       !
!                released during photosynthesis [mgC/mgO2]             !
!  CarbonToOxygenResp Mass ratio between carbon released and oxygen    !
!                consumed during respiration [mgC/mgO2]                !
!  TminRespiration Parameter of the exponent of the temperature        ! 
!                limiting function for respiration [1/degrees]         !
!  TminPhotosynthesis Parameter of the exponent of the temperature     ! 
!                limiting function for photosynthesis [1/degrees]      !
!  NitrogenLimitation Option for not including (0) or including N      !
!                limitation [nondimensional]                           !
!  PhosphorusLimitation Option for not including (0) or including P    !
!                limitation [nondimensional]                           !                  
!  pCO2air CO2 partial pressure in the air(parts per million by volume)!                                                                    
!                [parts per million by volume]                         !                                            
!  NitriR Nitrification rate: oxidation of NH4 to NO3 [1/day]          !                                                                     
!  kdenit Denitrification rate: reduction of NO3 to NH4 and N2 [1/day] !                                                                     
!  knitO2 Half-saturation constant for oxygen limitation of            ! 
!                nitrification [mg O2 L-1]                             !                                                                 
!  kdenitO2 Half-saturation constant for oxygen limitation of          !
!                nitrification [mg O2 L-1]                             !  
!  kt  Temperature limiting factor for mineralization,nitrification,   !
!                denitrification [Degrees centigrades-1]               ! 
!  minRate Mineralization rate of organic matter (in C, N and P)       !
!                [days-1]                                              !   
!  DenitNH4      Proportion of NH4 from                                !                      
!                denitrification [Dimensionless]                       !
!  MinToNH4      Fraction of dissolved organic nitrogen                !
!                that is mineralized to ammonia  [Dimensionless]       ! 
!  KI            Light intensity at which the                          ! 
!                inhibition of nitrification is half-saturated         !
!  I0            Light intensity threshold for                         !
!                light-inhibition of nitrification                     !                                
!                                                                      !   
!**********************************************************************!
!                                                                      !
!  ZooEED    Zooplankton excretion efficiency to Detritus pool,        !
!              {nondimensional].                                       !
!  ZooEEN    Zooplankton excretion efficiency to Nitrogen pool,        !
!              {nondimensional].                                       !
!  ZooGR     Zooplankton grazing rate, [1/day].                        !
!  ZooMRD    Zooplankton mortality rate to Detritus pool, [1/day].     !
!  ZooMRN    Zooplankton mortality rate to Nitrogen pool, [1/day].     !
!                                                                      !
!  LIGHTOBJ  Integer reference number of an EcoDynamo object to        !
!            calculate sea surface light intensity                     !
!  PHYOBJ1   Integer reference number of an EcoDynamo object to        !
!            simulate phytoplankton                                    !
!  PHYOBJ2   Integer reference number of an EcoDynamo object to        !
!            simulate phytoplankton                                    !
!  DISSOBJ1  Integer reference number of an EcoDynamo object to        !
!            simulate biogeochemniatry of dissolved matter             ! 
!  ICEALGALOBJ Integer reference number of an EcoDynamo object to      !
!              simulate ice algae                                      ! 
!                                                                      !
!=======================================================================
!
      USE mod_param
!
      implicit none
!
!  Set biological tracer identification indices.
!
      integer, allocatable :: idbio(:)  ! Biological tracers
      integer :: iNO3_                  ! Nitrate concentration
      integer :: iPhyt                  ! Phytoplankton concentration in carbon units (mmol C / m3)
      integer :: iZoop                  ! Zooplankton concentration
      integer :: iSDet                  ! Small detritus concentration
#ifdef ECODYNAMO
      integer :: Phyto_groups           ! Number of phytoplankton groups/taxa/species
      integer :: iNH4_                  ! Ammonia concentration     
      integer :: iPO4_                  ! Phosphate concentration 
      integer :: iPhyN                  ! Phytoplankton concentration in nitrogen units (mmol N / m3)
      integer :: iPhyP                  ! Phytoplankton concentration in phosphorus units (mmol P / m3)
      integer :: iSDeN                  ! Small detritus concentration in nitrogen units (mmol N / m3)
      integer :: iSDeP                  ! Small detritus concentration in phosphorus units (mmol P / m3)
      integer :: iZopN                  ! Zooplankton concentration in N units (mmol N / m3) 
      integer :: iZopP                  ! Zooplankton concentration in P units (mmol P / m3)
      integer :: iChlo                  ! Chlorophyll concentration (mg / m3) 
      integer :: iChld                  ! Chlorophyll concentration of diatoms (mg / m3)
      integer :: iTIC_                  ! Total inorganic carbon concentration (millimole/m3)
      integer :: iTAlk                  ! Total alkalinity (milliequivalents/m3)
      integer :: iOxyg                  ! Dissolved oxygen concentration (millimole/m3)
      integer :: iLphy                  ! Diatom concentration in carbon units (mmol C / m3) 
      integer :: iLphN                  ! Diatom concentration in nitrogen units (mmol N / m3) 
      integer :: iLphP                  ! Diatom concentration in phosphorus units (mmol P / m3) 
      integer :: iLphS                  ! Diatom concentration in Si units (mmol Si / m3) 
      
#endif  
!#ifdef CARBON
!      integer :: iLDeC                  ! Large detritus C-concentration
!      integer :: iSDeC                  ! Small detritus C-concentration
!      integer :: iTIC_                  ! Total inorganic carbon
!      integer :: iTAlk                  ! Total alkalinity
!#endif
!#ifdef OXYGEN
!      integer :: iOxyg                  ! Dissolved oxygen concentration
!#endif
!*************************Stuff imported from fennel_mod.h********************************************************
#if defined DIAGNOSTICS && defined DIAGNOSTICS_BIO
!
!  Biological 2D diagnostic variable IDs.
!
      integer, allocatable :: iDbio2(:)       ! 2D biological terms

      integer  :: iCOfx                       ! air-sea CO2 flux
      integer  :: iDNIT                       ! denitrification flux
      integer  :: ipCO2                       ! partial pressure of CO2
      integer  :: iO2fx                       ! air-sea O2 flux
!
!  Biological 3D diagnostic variable IDs.
!
      integer, allocatable :: iDbio3(:)       ! 3D biological terms

      integer  :: iPPro = 1                   ! primary productivity
      integer  :: iNO3u = 2                   ! NO3 uptake
#endif    
!**************************8***************************************************************************************
!  Biological parameters.
!
      integer, allocatable :: BioIter(:)

#ifdef ANA_BIOLOGY
      real(r8), allocatable :: BioIni(:,:)
#endif
      real(r8), allocatable :: AttPhy(:)       			! m2/mmole
      real(r8), allocatable :: AttSW(:)        			! 1/m
      real(r8), allocatable :: DetRR(:)        			! 1/day
      real(r8), allocatable :: K_NO3(:)        			! mmol/m3
      real(r8), allocatable :: Ivlev(:)        			! nondimensional
#ifdef ECODYNAMO
!****************************************EcoDynamo added stuff**************************************************************
      !real(r8), allocatable :: Chl2C_m(:,:)      	        ! nondimensional
      real(r8), allocatable :: Chl2C_m(:)      	                ! nondimensional
      real(r8), allocatable :: PARfrac(:)      			! nondimensional
      real(r8), allocatable :: Pmax(:)         			! Maximum photosynthetic rate (1/h)
      real(r8), allocatable :: Iopt(:)         			! Optimal light intensity (microEisntein/m2/s)
      real(r8), allocatable :: ImaxE(:)         		! Saturation light intensity (microEisntein/m2/s)
      real(r8), allocatable :: AEiler(:)       			! microEisntein/m2/s
      real(r8), allocatable :: BEiler(:)       			! microEisntein/m2/s
      real(r8), allocatable :: CEiler(:)       			! microEisntein/m2/s
      real(r8), allocatable :: MaintResp(:)    			! mmol O2/mg Chl/h
      real(r8), allocatable :: RespCoeff(:)    			! nondimensional
      real(r8), allocatable :: DocLoss(:)     			! nondimensional 
      real(r8), allocatable :: DocStressLoss(:)			! nondimensional 
      !real(r8), allocatable :: DeathLoss(:)    		! 1/day   See PhyMRD and PhyMRN below
      real(r8), allocatable :: RedfieldC(:)    			! gC/gP 
      real(r8), allocatable :: RedfieldN(:)    			! gN/gP 
      real(r8), allocatable :: RedfieldP(:)    			! gP/gP 
      real(r8), allocatable :: TemperatureAugmentationRate(:) 	! 1/degrees
      real(r8), allocatable :: RatioLightDarkRespiration(:)   	! nondimensional
      real(r8), allocatable :: minNPRatio(:)      	  	! mgN/mgP
      real(r8), allocatable :: maxNPRatio(:)      	   	! mgN/mgP
      real(r8), allocatable :: PMaxUptake(:)      	  	! 1/h
      real(r8), allocatable :: NMaxUptake(:)      	  	! 1/h
      real(r8), allocatable :: KP(:)              	  	! microM
      real(r8), allocatable :: KNO3(:)            	  	! microM See K_NO3 above 
      real(r8), allocatable :: KNH4(:)            	  	! microM
      real(r8), allocatable :: MinPCellQuota(:)   	  	! mgP/mgC
      real(r8), allocatable :: MaxPCellQuota(:)   	  	! mgP/mgC
      real(r8), allocatable :: MinNCellQuota(:)   	  	! mgN/mgC
      real(r8), allocatable :: MaxNCellQuota(:)   	  	! mgN/mgC
      real(r8), allocatable :: KPInternal(:)      	  	! mgP/mgC
      real(r8), allocatable :: KNInternal(:)              	! mgN/mgC
      !real(r8), allocatable :: SettlingSpeed(:)          	! m/day See wPhy above
      real(r8), allocatable :: CarbonToOxygenProd(:)      	! mgC/mgO2
      real(r8), allocatable :: CarbonToOxygenResp(:)      	! mgC/mgO2
      real(r8), allocatable :: TminRespiration(:)         	! degrees
      real(r8), allocatable :: TminPhotosynthesis(:)      	! degrees
      integer, allocatable :: NitrogenLimitation(:)   	        ! nondimensional
      integer, allocatable :: PhosphorusLimitation(:)           ! nondimensional
      real(r8), allocatable :: Beta(:)        			! mg C mg C-1 h-1 microM photons m2 s

      real(r8), allocatable :: Chl2C_mDiatoms(:)      			! nondimensional
      real(r8), allocatable :: PmaxDiatoms(:)         			! Maximum photosynthetic rate (1/h)
      real(r8), allocatable :: IoptDiatoms(:)         			! Optimal light intensity (microEisntein/m2/s)
      real(r8), allocatable :: ImaxEDiatoms(:)         		        ! Saturation light intensity (microEisntein/m2/s)
      real(r8), allocatable :: AEilerDiatoms(:)       			! microEisntein/m2/s
      real(r8), allocatable :: BEilerDiatoms(:)       			! microEisntein/m2/s
      real(r8), allocatable :: CEilerDiatoms(:)       			! microEisntein/m2/s
      real(r8), allocatable :: MaintRespDiatoms(:)    			! mmol O2/mg Chl/h
      real(r8), allocatable :: RespCoeffDiatoms(:)    			! nondimensional
      real(r8), allocatable :: DocLossDiatoms(:)     			! nondimensional 
      real(r8), allocatable :: DocStressLossDiatoms(:)			! nondimensional 
      real(r8), allocatable :: RedfieldCDiatoms(:)    			! gC/gP 
      real(r8), allocatable :: RedfieldNDiatoms(:)    			! gN/gP 
      real(r8), allocatable :: RedfieldPDiatoms(:)    			! gP/gP 
      real(r8), allocatable :: TemperatureAugmentationRateDiatoms(:) 	! 1/degrees
      real(r8), allocatable :: RatioLightDarkRespirationDiatoms(:)   	! nondimensional
      real(r8), allocatable :: minNPRatioDiatoms(:)      	  	! mgN/mgP
      real(r8), allocatable :: maxNPRatioDiatoms(:)      	   	! mgN/mgP
      real(r8), allocatable :: PMaxUptakeDiatoms(:)      	  	! 1/h
      real(r8), allocatable :: NMaxUptakeDiatoms(:)      	  	! 1/h
      real(r8), allocatable :: SiMaxUptake(:)      	  	        ! 1/h
      real(r8), allocatable :: KPDiatoms(:)              	  	! microM
      real(r8), allocatable :: K_NO3Diatoms(:)            	  	! microM 
      real(r8), allocatable :: KNH4Diatoms(:)            	  	! microM
      real(r8), allocatable :: MinPCellQuotaDiatoms(:)   	  	! mgP/mgC
      real(r8), allocatable :: MaxPCellQuotaDiatoms(:)   	  	! mgP/mgC
      real(r8), allocatable :: MinNCellQuotaDiatoms(:)   	  	! mgN/mgC
      real(r8), allocatable :: MaxNCellQuotaDiatoms(:)   	  	! mgN/mgC
      real(r8), allocatable :: KPInternalDiatoms(:)      	  	! mgP/mgC
      real(r8), allocatable :: KNInternalDiatoms(:)              	! mgN/mgC
      real(r8), allocatable :: KSiInternal(:)              	        ! mgSi/mgC
      real(r8), allocatable :: KSi(:)              	                ! microM
      real(r8), allocatable :: CarbonToOxygenProdDiatoms(:)      	! mgC/mgO2
      real(r8), allocatable :: CarbonToOxygenRespDiatoms(:)      	! mgC/mgO2
      real(r8), allocatable :: TminRespirationDiatoms(:)         	! degrees
      real(r8), allocatable :: TminPhotosynthesisDiatoms(:)      	! degrees
      integer, allocatable :: NitrogenLimitationDiatoms(:)   	        ! nondimensional
      integer, allocatable :: PhosphorusLimitationDiatoms(:)            ! nondimensional
      integer, allocatable :: SilicaLimitation(:)                       ! nondimensional
      real(r8), allocatable :: PhyISDiatoms(:)        			! mg C mg C-1 h-1 microM photons m2 s
      real(r8), allocatable :: BetaDiatoms(:)        			! mg C mg C-1 h-1 microM photons m2 s
      real(r8), allocatable :: PhyMRDDiatoms(:)       			! 1/day
      real(r8), allocatable :: PhyMRNDiatoms(:)       			! 1/day
      real(r8), allocatable :: Vm_NO3Diatoms(:)       			! 1/day
      real(r8), allocatable :: MaxSiCellQuota(:)                        ! mgSi/mgC
      real(r8), allocatable :: minNSiRatio(:)                           ! mgN/mgSi
      real(r8), allocatable :: RedfieldSi(:)                            ! gSi/gP
       
      real(r8), allocatable :: NitriR(:)            	  	! 1/day 
      real(r8), allocatable :: kdenit(:)            	  	! 1/day 
      real(r8), allocatable :: knitO2(:)            	  	! mg/L
      real(r8), allocatable :: kdenitO2(:)            	  	! mg/L
      real(r8), allocatable :: pCO2air(:)            	  	! parts per million by volume
      real(r8), allocatable :: kt(:)            	  	! Degrees centigrades-1
      real(r8), allocatable :: minRateC(:)            	  	! Days-1
      real(r8), allocatable :: minRateN(:)            	  	! Days-1
      real(r8), allocatable :: minRateP(:)            	  	! Days-1
      real(r8), allocatable :: DenitNH4(:)                      ! Dimensionless
      real(r8), allocatable :: MinToNH4(:)                      ! Dimensionless
      real(r8), allocatable :: KI(:)                            ! W m-2
      real(r8), allocatable :: I0(:)                            ! W m-2
!***************************************************************************************************************************
#endif
#ifdef TANGENT
      real(r8), allocatable :: tl_PARfrac(:)
#endif
#ifdef ADJOINT
      real(r8), allocatable :: ad_PARfrac(:)
#endif
      real(r8), allocatable :: PhyIS(:)        			! mg C mg C-1 h-1 microM photons m2 s
      real(r8), allocatable :: PhyMRD(:)       			! 1/day
      real(r8), allocatable :: PhyMRN(:)       			! 1/day
      real(r8), allocatable :: Vm_NO3(:)       			! 1/day
      real(r8), allocatable :: wDet(:)         			! m/day
#ifdef TANGENT
      real(r8), allocatable :: tl_wDet(:)
#endif
#ifdef ADJOINT
      real(r8), allocatable :: ad_wDet(:)
#endif
      real(r8), allocatable :: wPhy(:)         ! m/day
#ifdef TANGENT
      real(r8), allocatable :: tl_wPhy(:)
#endif
#ifdef ADJOINT
      real(r8), allocatable :: ad_wPhy(:)
#endif

      real(r8), allocatable :: wPhyDiatoms(:)  ! m/day
#ifdef TANGENT
      real(r8), allocatable :: tl_wPhyDiatoms(:)
#endif
#ifdef ADJOINT
      real(r8), allocatable :: ad_wPhyDiatoms(:)
#endif
      real(r8), allocatable :: ZooEED(:)       ! nondimensional
      real(r8), allocatable :: ZooEEN(:)       ! nondimensional
      real(r8), allocatable :: ZooGR(:)        ! 1/day
      real(r8), allocatable :: ZooMRD(:)       ! 1/day
      real(r8), allocatable :: ZooMRN(:)       ! 1/day

!  EcoDynamo object integer references
      integer(kind=8) :: LIGHTOBJ, LIGHTOBJM 
      integer(kind=8) :: PHYOBJ1
      integer(kind=8) :: PHYOBJ2  
      integer(kind=8) :: DISSOBJ1
      integer(kind=8) :: ICEALGALOBJ

      CONTAINS

      SUBROUTINE initialize_biology
!
!=======================================================================
!                                                                      !
!  This routine sets several variables needed by the biology model.    !
!  It allocates and assigns biological tracers indices.                !
!                                                                      !
!=======================================================================
!
!  Local variable declarations
!
      integer :: i, ic
!
!-----------------------------------------------------------------------
!  Set number of biological tracers.
!-----------------------------------------------------------------------
!


#ifdef ECODYNAMO
      NBT=17
      Phyto_groups = 2
#else
      NBT=4
      Phyto_groups = 1
#endif
!*************************Stuff imported from fennel_mod.h********************************************************
#if defined DIAGNOSTICS && defined DIAGNOSTICS_BIO
!
!-----------------------------------------------------------------------
!  Set sources and sinks biology diagnostic parameters.
!-----------------------------------------------------------------------
!
!  Set number of diagnostics terms.
!
      NDbio3d=2
      NDbio2d=0
# ifdef DENITRIFICATION
      NDbio2d=NDbio2d+1
# endif
# ifdef CARBON
      NDbio2d=NDbio2d+2
# endif
# ifdef OXYGEN
      NDbio2d=NDbio2d+1
# endif
!
!  Initialize biology diagnostic indices.
!
      ic=0
# ifdef DENITRIFICATION
      ic=ic+1 
      iDNIT=ic
# endif
# ifdef CARBON
      ic=ic+1 
      iCOfx=ic
      ic=ic+1 
      ipCO2=ic
# endif
# ifdef OXYGEN
      ic=ic+1 
      iO2fx=ic
# endif
#endif
!************************************************************************************************************
!-----------------------------------------------------------------------
!  Allocate various module variables.
!-----------------------------------------------------------------------
!
      IF (.not.allocated(BioIter)) THEN
        allocate ( BioIter(Ngrids) )
      END IF
      IF (.not.allocated(AttPhy)) THEN
        allocate ( AttPhy(Ngrids) )
      END IF
      IF (.not.allocated(AttSW)) THEN
        allocate ( AttSW(Ngrids) )
      END IF
      IF (.not.allocated(DetRR)) THEN
        allocate ( DetRR(Ngrids) )
      END IF
      IF (.not.allocated(K_NO3)) THEN
        allocate ( K_NO3(Ngrids) )
      END IF
      IF (.not.allocated(Chl2C_m)) THEN
        !allocate ( Chl2C_m(Phyto_groups,Ngrids))
         allocate ( Chl2C_m(Ngrids))
      END IF
      IF (.not.allocated(Ivlev)) THEN
        allocate ( Ivlev(Ngrids) )
      END IF
      IF (.not.allocated(PARfrac)) THEN
        allocate ( PARfrac(Ngrids) )
      END IF
#ifdef TANGENT
      IF (.not.allocated(tl_PARfrac)) THEN
        allocate ( tl_PARfrac(Ngrids) )
      END IF
#endif
#ifdef ADJOINT
      IF (.not.allocated(ad_PARfrac)) THEN
        allocate ( ad_PARfrac(Ngrids) )
      END IF
#endif
      IF (.not.allocated(PhyIS)) THEN
        allocate ( PhyIS(Ngrids) )
      END IF
      IF (.not.allocated(Beta)) THEN
        allocate ( Beta(Ngrids) )
      END IF
      IF (.not.allocated(PhyMRD)) THEN
        allocate ( PhyMRD(Ngrids) )
      END IF
      IF (.not.allocated(PhyMRN)) THEN
        allocate ( PhyMRN(Ngrids) )
      END IF
      IF (.not.allocated(Vm_NO3)) THEN
        allocate ( Vm_NO3(Ngrids) )
      END IF
      IF (.not.allocated(wDet)) THEN
        allocate ( wDet(Ngrids) )
      END IF
      IF (.not.allocated(Pmax)) THEN
        allocate ( Pmax(Ngrids) )
      END IF
      IF (.not.allocated(Iopt)) THEN
        allocate ( Iopt(Ngrids) )
      END IF
      IF (.not.allocated(ImaxE)) THEN
        allocate ( ImaxE(Ngrids) )
      END IF
      IF (.not.allocated(AEiler)) THEN
        allocate ( AEiler(Ngrids) )
      END IF
      IF (.not.allocated(BEiler)) THEN
        allocate ( BEiler(Ngrids) )
      END IF
      IF (.not.allocated(CEiler)) THEN
        allocate ( CEiler(Ngrids) )
      END IF
      IF (.not.allocated(MaintResp)) THEN
        allocate ( MaintResp(Ngrids) )
      END IF
      IF (.not.allocated(RespCoeff)) THEN
        allocate ( RespCoeff(Ngrids) )
      END IF
      IF (.not.allocated(DocLoss)) THEN
        allocate ( DocLoss(Ngrids) )
      END IF
      IF (.not.allocated(DocStressLoss)) THEN
        allocate ( DocStressLoss(Ngrids) )
      END IF
      IF (.not.allocated(RedfieldC)) THEN
        allocate ( RedfieldC(Ngrids) )
      END IF
      IF (.not.allocated(RedfieldN)) THEN
        allocate ( RedfieldN(Ngrids) )
      END IF
      IF (.not.allocated(RedfieldP)) THEN
        allocate ( RedfieldP(Ngrids) )
      END IF
      IF (.not.allocated(TemperatureAugmentationRate)) THEN
        allocate ( TemperatureAugmentationRate(Ngrids) )
      END IF
      IF (.not.allocated(RatioLightDarkRespiration)) THEN
        allocate ( RatioLightDarkRespiration(Ngrids) )
      END IF
      IF (.not.allocated(minNPRatio)) THEN
        allocate (minNPRatio(Ngrids) )
      END IF
      IF (.not.allocated(maxNPRatio)) THEN
        allocate (maxNPRatio(Ngrids) )
      END IF
      IF (.not.allocated(PMaxUptake)) THEN
        allocate (PMaxUptake(Ngrids) )
      END IF
      IF (.not.allocated(NMaxUptake)) THEN
        allocate (NMaxUptake(Ngrids) )
      END IF
      IF (.not.allocated(KP)) THEN
        allocate (KP(Ngrids) )
      END IF
      IF (.not.allocated(KNH4)) THEN
        allocate (KNH4(Ngrids) )
      END IF
      IF (.not.allocated(MinPCellQuota)) THEN
        allocate (MinPCellQuota(Ngrids) )
      END IF
      IF (.not.allocated(MaxPCellQuota)) THEN
        allocate (MaxPCellQuota(Ngrids) )
      END IF
      IF (.not.allocated(MinNCellQuota)) THEN
        allocate (MinNCellQuota(Ngrids) )
      END IF
      IF (.not.allocated(MaxNCellQuota)) THEN
        allocate (MaxNCellQuota(Ngrids) )
      END IF
      IF (.not.allocated(KPInternal)) THEN
        allocate (KPInternal(Ngrids) )
      END IF
      IF (.not.allocated(KNInternal)) THEN
        allocate (KNInternal(Ngrids) )
      END IF
      IF (.not.allocated(CarbonToOxygenProd)) THEN
        allocate (CarbonToOxygenProd(Ngrids) )
      END IF
      IF (.not.allocated(CarbonToOxygenResp)) THEN
        allocate (CarbonToOxygenResp(Ngrids) )
      END IF
      IF (.not.allocated(TminRespiration)) THEN
        allocate (TminRespiration(Ngrids) )
      END IF
      IF (.not.allocated(TminPhotosynthesis)) THEN
        allocate (TminPhotosynthesis(Ngrids) )
      END IF
      IF (.not.allocated(NitrogenLimitation)) THEN
        allocate (NitrogenLimitation(Ngrids) )
      END IF
      IF (.not.allocated(PhosphorusLimitation)) THEN
        allocate (PhosphorusLimitation(Ngrids) )
      END IF
      IF (.not.allocated(NitriR)) THEN
        allocate (NitriR(Ngrids) )
      END IF
      IF (.not.allocated(kdenit)) THEN
        allocate (kdenit(Ngrids) )
      END IF
      IF (.not.allocated(knitO2)) THEN
        allocate (knitO2(Ngrids) )
      END IF
      IF (.not.allocated(kdenitO2)) THEN
        allocate (kdenitO2(Ngrids) )
      END IF
      IF (.not.allocated(pCO2air)) THEN
        allocate (pCO2air(Ngrids) )
      END IF
      IF (.not.allocated(kt)) THEN
        allocate (kt(Ngrids) )
      END IF
      IF (.not.allocated(minRateC)) THEN
        allocate (minRateC(Ngrids) )
      END IF 
      IF (.not.allocated(minRateN)) THEN
        allocate (minRateN(Ngrids) )
      END IF 
      IF (.not.allocated(minRateP)) THEN
        allocate (minRateP(Ngrids) )
      END IF 
      IF (.not.allocated(DenitNH4)) THEN
        allocate (DenitNH4(Ngrids) )
      END IF 
      IF (.not.allocated(MinToNH4)) THEN
        allocate (MinToNH4(Ngrids) )
      END IF 
      IF (.not.allocated(KI)) THEN
        allocate (KI(Ngrids) )
      END IF 
      IF (.not.allocated(I0)) THEN
        allocate (I0(Ngrids) )
      END IF 
#ifdef TANGENT
      IF (.not.allocated(tl_wDet)) THEN
        allocate ( tl_wDet(Ngrids) )
      END IF
#endif
#ifdef ADJOINT
      IF (.not.allocated(ad_wDet)) THEN
        allocate ( ad_wDet(Ngrids) )
      END IF
#endif
      IF (.not.allocated(wPhy)) THEN
        allocate ( wPhy(Ngrids) )
      END IF
#ifdef TANGENT
      IF (.not.allocated(tl_wPhy)) THEN
        allocate ( tl_wPhy(Ngrids) )
      END IF
#endif
#ifdef ADJOINT
      IF (.not.allocated(ad_wPhy)) THEN
        allocate ( ad_wPhy(Ngrids) )
      END IF
#endif
      IF (.not.allocated(ZooEED)) THEN
        allocate ( ZooEED(Ngrids) )
      END IF
      IF (.not.allocated(ZooEEN)) THEN
        allocate ( ZooEEN(Ngrids) )
      END IF
      IF (.not.allocated(ZooGR)) THEN
        allocate ( ZooGR(Ngrids) )
      END IF
      IF (.not.allocated(ZooMRD)) THEN
        allocate ( ZooMRD(Ngrids) )
      END IF
      IF (.not.allocated(ZooMRN)) THEN
        allocate ( ZooMRN(Ngrids) )
      END IF

      IF (.not.allocated(Chl2C_mDiatoms)) THEN
        allocate ( Chl2C_mDiatoms(Ngrids) )
      END IF  
      IF (.not.allocated(PmaxDiatoms)) THEN
        allocate ( PmaxDiatoms(Ngrids) )
      END IF 
      IF (.not.allocated(IoptDiatoms)) THEN
        allocate ( IoptDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(ImaxEDiatoms)) THEN
        allocate ( ImaxEDiatoms(Ngrids) )
      END IF 
      IF (.not.allocated(AEilerDiatoms)) THEN
        allocate ( AEilerDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(BEilerDiatoms)) THEN
        allocate ( BEilerDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(CEilerDiatoms)) THEN
        allocate ( CEilerDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(MaintRespDiatoms)) THEN
        allocate ( MaintRespDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(RespCoeffDiatoms)) THEN
        allocate ( RespCoeffDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(DocLossDiatoms)) THEN
        allocate ( DocLossDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(DocStressLossDiatoms)) THEN
        allocate ( DocStressLossDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(RedfieldCDiatoms)) THEN
        allocate ( RedfieldCDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(RedfieldNDiatoms)) THEN
        allocate ( RedfieldNDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(RedfieldPDiatoms)) THEN
        allocate ( RedfieldPDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(TemperatureAugmentationRateDiatoms)) THEN
        allocate ( TemperatureAugmentationRateDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(RatioLightDarkRespirationDiatoms)) THEN
        allocate ( RatioLightDarkRespirationDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(minNPRatioDiatoms)) THEN
        allocate ( minNPRatioDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(maxNPRatioDiatoms)) THEN
        allocate ( maxNPRatioDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(PMaxUptakeDiatoms)) THEN
        allocate ( PMaxUptakeDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(NMaxUptakeDiatoms)) THEN
        allocate ( NMaxUptakeDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(SiMaxUptake)) THEN
        allocate ( SiMaxUptake(Ngrids) )
      END IF 
      IF (.not.allocated(RedfieldSi)) THEN
        allocate ( RedfieldSi(Ngrids) )
      END IF 
      IF (.not.allocated(KPDiatoms)) THEN
        allocate ( KPDiatoms(Ngrids) )
      END IF 
      IF (.not.allocated(K_NO3Diatoms)) THEN
        allocate ( K_NO3Diatoms(Ngrids) )
      END IF 
      IF (.not.allocated(KNH4Diatoms)) THEN
        allocate ( KNH4Diatoms(Ngrids) )
      END IF 
      IF (.not.allocated(MinPCellQuotaDiatoms)) THEN
        allocate ( MinPCellQuotaDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(MaxPCellQuotaDiatoms)) THEN
        allocate ( MaxPCellQuotaDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(MinNCellQuotaDiatoms)) THEN
        allocate ( MinNCellQuotaDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(MaxNCellQuotaDiatoms)) THEN
        allocate ( MaxNCellQuotaDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(KPInternalDiatoms)) THEN
        allocate ( KPInternalDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(KNInternalDiatoms)) THEN
        allocate ( KNInternalDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(KSiInternal)) THEN
        allocate ( KSiInternal(Ngrids) )
      END IF
      IF (.not.allocated(KSi)) THEN
        allocate ( KSi(Ngrids) )
      END IF
      IF (.not.allocated(CarbonToOxygenProdDiatoms)) THEN
        allocate ( CarbonToOxygenProdDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(CarbonToOxygenRespDiatoms)) THEN
        allocate ( CarbonToOxygenRespDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(TminRespirationDiatoms)) THEN
        allocate ( TminRespirationDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(TminPhotosynthesisDiatoms)) THEN
        allocate ( TminPhotosynthesisDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(NitrogenLimitationDiatoms)) THEN
        allocate ( NitrogenLimitationDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(PhosphorusLimitationDiatoms)) THEN
        allocate ( PhosphorusLimitationDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(SilicaLimitation)) THEN
        allocate ( SilicaLimitation(Ngrids) )
      END IF
      IF (.not.allocated(PhyISDiatoms)) THEN
        allocate ( PhyISDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(BetaDiatoms)) THEN
        allocate ( BetaDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(PhyMRDDiatoms)) THEN
        allocate ( PhyMRDDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(PhyMRNDiatoms)) THEN
        allocate ( PhyMRNDiatoms(Ngrids) )
      END IF
      IF (.not.allocated(Vm_NO3Diatoms)) THEN
        allocate ( Vm_NO3Diatoms(Ngrids) )
      END IF
      IF (.not.allocated(MaxSiCellQuota)) THEN
        allocate ( MaxSiCellQuota(Ngrids) )
      END IF 
      IF (.not.allocated(minNSiRatio)) THEN
        allocate ( minNSiRatio(Ngrids) )
      END IF 
      IF (.not.allocated(wPhyDiatoms)) THEN
        allocate ( wPhyDiatoms(Ngrids) )
      END IF
#ifdef TANGENT
      IF (.not.allocated(tl_wPhyDiatoms)) THEN
        allocate ( tl_wPhyDiatoms(Ngrids) )
      END IF
#endif
#ifdef ADJOINT
      IF (.not.allocated(ad_wPhyDiatoms)) THEN
        allocate ( ad_wPhyDiatoms(Ngrids) )
      END IF
#endif
!
!  Allocate biological tracer vector.
!
      IF (.not.allocated(idbio)) THEN
        allocate ( idbio(NBT) )
      END IF
!*******************Stuff imported from fennel_mod.h****************************************************
#if defined DIAGNOSTICS && defined DIAGNOSTICS_BIO
!
!  Allocate biological diagnostics vectors
!
      IF (.not.allocated(iDbio2)) THEN
        allocate ( iDbio2(NDbio2d) )
      END IF
      IF (.not.allocated(iDbio3)) THEN
        allocate ( iDbio3(NDbio3d) )
      END IF
#endif
!********************************************************************************************************
!-----------------------------------------------------------------------
!  Initialize tracer identification indices.
!-----------------------------------------------------------------------
!
      ic=NAT+NPT+NCS+NNS
      DO i=1,NBT
        idbio(i)=ic+i
      END DO
      iNO3_=ic+1
      iPhyt=ic+2
      iZoop=ic+3
      iSDet=ic+4
#ifdef ECODYNAMO
      iNH4_=ic+5                       
      iPO4_=ic+6                  
      iPhyN=ic+7                
      iPhyP=ic+8                
      iSDeN=ic+9                 
      iSDeP=ic+10
      iZopN=ic+11
      iZopP=ic+12
      iChlo=ic+13
      iTIC_=ic+14
      iTAlk=ic+15
      iOxyg=ic+16
      iLphN=ic+17
   

#endif
      LIGHTOBJ = 0
      LIGHTOBJM = 0
      PHYOBJ1 = 0
      PHYOBJ2 = 0
      DISSOBJ1 = 0
      ICEALGALOBJ = 0
      RETURN

  END SUBROUTINE initialize_biology
